;*******************************************************************************
; Pulse symmetry tester
; This program allows to test symmetry of pulses of Czechoslovak Turbo 2000,
; Polish Turbo Blizzard and compatible systems.
; 
; The yellow and pink vertical lines should keep the same distance from
; the central white vertical line
;
; Assemble with MADS 

; Version history:
; 2019-05-14 1.0 Initial version 
; 
;*******************************************************************************
         ORG 8192
         ICL 'equates.asm'
         
;===============================================================================
; Intro screen
;===============================================================================

         sei               ;Setup display list and screen memory
         lda #<DL 
         sta SDLSTL 
         lda #>DL 
         sta SDLSTH 
         cli
         
         lda #180          ;Setup colors   
         sta COLOR2
         lda #12
         sta COLOR1
         
         lda #255          ;Wait for a key to be pressed
         sta CH
KBLOOP   lda CH
         cmp #255
         beq KBLOOP
         
         jmp INIT

;-------------------------------------------------------------------------------
; Screen
;-------------------------------------------------------------------------------                         
         
DL      .BYTE 112,112,112,2+64,<SCMEM,>SCMEM,2,112,2,65,<DL,>DL
;            1234567890123456789012345678901234567890
SCMEM   .SB '          PULSE SYMMETRY TEST           '
        .SB '   for Turbo 2000 and Turbo Blizzard    '
        .SB '        Press any key to begin          '           

;===============================================================================
; Initialize
;===============================================================================
INIT     lda #1
         sta COLDST        ;Allow cold start after RESET
         
         lda #52           ;Switch program recorder to turbo mode                                                             
         sta PACTL         ;Motor ON
         sta PBCTL         ;Command active  (Turbo 2000 ON)
         
         lda #$83          ;DATA OUT forced to zero (Turbo Blizzard ON)
         sta SKCTL 
            
         lda #128          ;Disable interrupts and DMA
         sta POKMSK
         sta IRQEN
            
         lda #0
         sta NMIEN
         sta DMACLT
;===============================================================================
; Set PMG
;===============================================================================

         lda #$0C          ;Set PMG colors
         sta COLPM0
         lda #$1C
         sta COLPM1
         lda #$4C
         sta COLPM2
         
         lda #127          ;Set PMG initial locations
         sta HPOSP0
         sta HPOSP1
         sta HPOSP2
         
         lda #16           ;Set graphic patterns of PMGs
         sta GRAFP0
         lda #48
         sta GRAFP1
         sta GRAFP2
         
;===============================================================================
; Main loop
;===============================================================================
LOOP     lda SKSTAT         ;Get serial port
         and #16            ;Mask bit status
         sta COLBK          ;Background color
         cmp PREV           ;Is it as previous?
         beq SAME           ;Yes, go increase counter
         tax                ;Keep value in X
;-------------------------------------------------------------------------------
; Values differ, we reached edge
;-------------------------------------------------------------------------------         
DIFF     lda PREV           ;Check the original value
         beq WAS_LO         
         
WAS_HI   lda NUM_HI         ;Get count
         clc                ;Quadruple the count
         asl        
         asl       
         clc
         adc #127           ;Display as player 1
         sta HPOSP1
         nop                ;Compensation NOPs
         nop
         jmp SET_END           

WAS_LO   lda NUM_LO         ;Get count
         clc                ;Quadruple the count
         asl    
         asl                
         sta NUM_LO
         lda #127
         sec
         sbc NUM_LO
         sta HPOSP2         ;Display as player 2      

SET_END  stx PREV           ;New previous SKSTAT value
         ldx #0             ;Clear counters
         stx NUM_LO
         stx NUM_HI
         
         jmp LOOP           ;Loop again
;-------------------------------------------------------------------------------
; The value is just the same, increase the appropriate counter
;-------------------------------------------------------------------------------         
SAME     lda PREV           ;What is the value
         bne CNT_HI         ;Logical 1, increase HIGH
CNT_LO   inc NUM_LO         ;Logical 0, increase LOW
         jmp LOOP                  
         
CNT_HI   inc NUM_HI           
         jmp LOOP

;===============================================================================
;Data area
;===============================================================================
NUM_LO  .BYTE 0
NUM_HI  .BYTE 0
PREV    .BYTE 0

;===============================================================================
; RUN segment
;===============================================================================
         ORG RUNAD
         .WORD 8192